@@ -41,19 +41,16 @@
   *
   **/
 void gnutls_credentials_clear( gnutls_session session) {
-	AUTH_CRED * ccred, *ncred;
-	
-	if (session->key && session->key->cred) { /* begining of the list */
+	if (session->key && session->key->cred) { /* beginning of the list */
+		AUTH_CRED * ccred, *ncred;
 		ccred = session->key->cred;
 		while(ccred!=NULL) {
 			ncred = ccred->next;
-			if (ccred!=NULL) gnutls_free(ccred);
+			gnutls_free(ccred);
 			ccred = ncred;
 		}
 		session->key->cred = NULL;
 	}
-
-	return;
 }
 
 /* 
@@ -89,12 +86,12 @@ int gnutls_credentials_set( gnutls_session session, gnutls_credentials_type type
 	AUTH_CRED * ccred=NULL, *pcred=NULL;
 	int exists=0;	
 	
-	if (session->key->cred==NULL) { /* begining of the list */
+	if (session->key->cred==NULL) { /* beginning of the list */
 		
 		session->key->cred = gnutls_malloc(sizeof(AUTH_CRED));
 		if (session->key->cred == NULL) return GNUTLS_E_MEMORY_ERROR;
 		
-		/* copy credentials localy */
+		/* copy credentials locally */
 		session->key->cred->credentials = cred;
 		
 		session->key->cred->next = NULL;
@@ -118,7 +115,7 @@ int gnutls_credentials_set( gnutls_session session, gnutls_credentials_type type
 		
 			ccred = pcred->next;
 
-			/* copy credentials localy */
+			/* copy credentials locally */
 			ccred->credentials = cred;
 
 			ccred->next = NULL;
@@ -156,7 +153,7 @@ int server = session->security_parameters.entity==GNUTLS_SERVER?0:1;
 }
 
 /* 
- * This returns an pointer to the linked list. Don't
+ * This returns a pointer to the linked list. Don't
  * free that!!!
  */
 const void *_gnutls_get_kx_cred( gnutls_session session, gnutls_kx_algorithm algo, int *err) 
@@ -167,10 +164,12 @@ int server = session->security_parameters.entity==GNUTLS_SERVER?1:0;
 }
 
 const void *_gnutls_get_cred( GNUTLS_KEY key, gnutls_credentials_type type, int *err) {
+	const void *retval = NULL;
+	int _err = -1;
 	AUTH_CRED * ccred;
 
-	if (key == NULL) return NULL;
-	
+	if (key == NULL) goto out;
+
 	ccred = key->cred;
 	while(ccred!=NULL) {
 		if (ccred->algorithm==type) {
@@ -178,13 +177,14 @@ const void *_gnutls_get_cred( GNUTLS_KEY key, gnutls_credentials_type type, int
 		}
 		ccred = ccred->next;
 	}
-	if (ccred==NULL) {
-		if (err!=NULL) *err=-1;
-		return NULL;
-	}
-			
-	if (err!=NULL) *err=0;
-	return ccred->credentials;
+	if (ccred==NULL) goto out;
+
+	_err = 0;
+	retval = ccred->credentials;
+
+	out:
+	if (err!=NULL) *err=_err;
+	return retval;
 }
 
 /*-
@@ -208,7 +208,7 @@ void* _gnutls_get_auth_info( gnutls_session session) {
   * _gnutls_free_auth_info - Frees the auth info structure
   * @session: is a &gnutls_session structure.
   *
-  * this function frees the auth info structure and sets it to
+  * This function frees the auth info structure and sets it to
   * null. It must be called since some structures contain malloced
   * elements.
   -*/
