@@ -29,8 +29,10 @@
 #define GNUTLS_E_EXPIRED -29
 #define GNUTLS_E_DB_ERROR -30
 #define GNUTLS_E_SRP_PWD_ERROR -31
-#define GNUTLS_E_INSUFICIENT_CREDENTIALS -32
-#define GNUTLS_E_INSUFICIENT_CRED GNUTLS_E_INSUFICIENT_CREDENTIALS
+#define GNUTLS_E_INSUFFICIENT_CREDENTIALS -32
+#define GNUTLS_E_INSUFICIENT_CREDENTIALS GNUTLS_E_INSUFFICIENT_CREDENTIALS /* for backwards compatibility only */
+#define GNUTLS_E_INSUFFICIENT_CRED GNUTLS_E_INSUFFICIENT_CREDENTIALS
+#define GNUTLS_E_INSUFICIENT_CRED GNUTLS_E_INSUFFICIENT_CREDENTIALS /* for backwards compatibility only */
 
 #define GNUTLS_E_HASH_FAILED -33
 #define GNUTLS_E_BASE64_DECODING_ERROR -34
@@ -115,10 +117,13 @@
 #define GNUTLS_E_INVALID_PASSWORD -99
 #define GNUTLS_E_MAC_VERIFY_FAILED -100 /* for PKCS #12 MAC */
 
-#define GNUTLS_E_UNIMPLEMENTED_FEATURE -250
+#define GNUTLS_E_BASE64_ENCODING_ERROR -201
+#define GNUTLS_E_INCOMPATIBLE_GCRYPT_LIBRARY -202
+
+#define GNUTLS_E_UNIMPLEMENTED_FEATURE -1250
 
 /* _INT_ internal errors. Not exported */
 
-#define GNUTLS_E_INT_RET_0 -251
+#define GNUTLS_E_INT_RET_0 -1251
 
 #endif /* GNUTLS_ERRORS_IH */
