@@ -7,7 +7,9 @@ int _gnutls_x509_read_dsa_pubkey(opaque * der, int dersize, GNUTLS_MPI * params)
 int _gnutls_x509_read_dsa_params(opaque * der, int dersize, GNUTLS_MPI * params);
 
 int _gnutls_x509_write_rsa_params( GNUTLS_MPI * params, int params_size,
-	opaque * der, int* dersize);
+	gnutls_datum* der);
+int _gnutls_x509_write_dsa_params( GNUTLS_MPI * params, int params_size,
+        gnutls_datum* der);
 
 int _gnutls_x509_read_uint( ASN1_TYPE node, const char* value, 
 	unsigned int* ret);
