@@ -35,7 +35,7 @@
 #include <sys/time.h>
 #include <signal.h>
 #include <netdb.h>
-#include <common.h>
+#include "common.h"
 #include "cli-gaa.h"
 
 #ifndef SHUT_WR
@@ -222,7 +222,7 @@ static int handle_error(socket_st hd, int err)
 	}
 
 	str = gnutls_strerror(err);
-	if (str == NULL) str = "(unknown)";
+	if (str == NULL) str = str_unknown;
 	fprintf(stderr,
 		"*** %s error: %s\n", err_type, str);
 
@@ -230,7 +230,7 @@ static int handle_error(socket_st hd, int err)
 	    || err == GNUTLS_E_FATAL_ALERT_RECEIVED) {
 		alert = gnutls_alert_get(hd.session);
 		str = gnutls_alert_get_name(alert);
-		if (str == NULL) str = "(unknown)";
+		if (str == NULL) str = str_unknown;
 		printf("*** Received alert [%d]: %s\n", alert, str);
 
 	}
@@ -240,7 +240,7 @@ static int handle_error(socket_st hd, int err)
 	return ret;
 }
 
-int starttls_alarmed;
+int starttls_alarmed = 0;
 
 void starttls_alarm (int signum)
 {
@@ -378,8 +378,13 @@ int main(int argc, char **argv)
 
 	signal (SIGALRM, &starttls_alarm);
 
-	FD_ZERO(&rset);
+	/* do not buffer */
+	setbuf(stdin, NULL);
+	setbuf(stdout, NULL);
+	setbuf(stderr, NULL);
+
 	for (;;) {
+		FD_ZERO(&rset);
 		FD_SET(fileno(stdin), &rset);
 		FD_SET(sd, &rset);
 
@@ -727,6 +732,7 @@ static void init_global_tls_stuff(void)
 		}
 	}
 
+#ifdef HAVE_LIBOPENCDK
 	if (pgp_certfile != NULL) {
 		ret =
 		    gnutls_certificate_set_openpgp_key_file(xcred,
@@ -758,6 +764,8 @@ static void init_global_tls_stuff(void)
 				"Error setting the OpenPGP trustdb file\n");
 		}
 	}
+#endif
+
 #ifdef ENABLE_SRP
 	/* SRP stuff */
 	if (gnutls_srp_allocate_client_credentials(&srp_cred) < 0) {
