@@ -18,6 +18,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include <config.h>
 #include <stdio.h>
 #include <errno.h>
 #include <stdlib.h>
@@ -53,6 +54,7 @@ char *hostname=NULL;
 int port;
 int record_max_size;
 int fingerprint;
+static int debug;
 
 gnutls_srp_client_credentials srp_cred;
 gnutls_anon_client_credentials anon_cred;
@@ -66,6 +68,11 @@ int more_info = 0;
 int tls1_ok = 0;
 int ssl3_ok = 0;
 
+static void tls_log_func( int level, const char* str)
+{
+	fprintf(stderr, "|<%d>| %s", level, str);
+}
+
 typedef int (*TEST_FUNC)( gnutls_session);
 
 typedef struct {
@@ -93,6 +100,8 @@ static const TLS_TEST tls_tests[] = {
 	{ "for certificate information", test_certificate, "", "", "" },
 	{ "for trusted CAs", test_server_cas, "", "", "" },
 	{ "whether the server understands TLS closure alerts", test_bye, "yes", "no", "partially"},
+	/* the fact that is after the closure alert test matter.
+	 */
 	{ "whether the server supports session resumption", test_session_resume2, "yes", "no", "dunno"},
 	{ "for export-grade ciphersuite support", test_export, "yes", "no", "dunno" },
 #ifdef ENABLE_ANON
@@ -124,7 +133,7 @@ static int tt = 0;
 		sa.sin_port = htons(port); \
 		sa.sin_addr.s_addr = *((unsigned int *) server_host->h_addr); \
 		inet_ntop(AF_INET, &sa.sin_addr, buffer, MAX_BUF); \
-		if (tt++ == 0) fprintf(stderr, "Connecting to '%s:%d'...\n", buffer, port); \
+		if (tt++ == 0) printf("Connecting to '%s:%d'...\n", buffer, port); \
 		err = connect(sd, (SA *) & sa, sizeof(sa)); \
 		ERR(err, "connect")
 
@@ -150,6 +159,9 @@ int main(int argc, char **argv)
 		exit(1);
 	}
 
+	gnutls_global_set_log_function( tls_log_func);
+	gnutls_global_set_log_level(debug);
+
 	if (gnutls_global_init_extra() < 0) {
 		fprintf(stderr, "global state initialization error\n");
 		exit(1);
@@ -242,6 +254,8 @@ void gaa_parser(int argc, char **argv)
 	if (info.rest_args==NULL) hostname="localhost";
 	else hostname = info.rest_args;
 
+	debug = info.debug;
+
 	more_info = info.more_info;
 	
 }
